/*
 * File: LetterFrequency.java
 * --------------------------
 * This program builds a table showing the frequency of letters in
 * a data file.
 */

package demos;

import acm.program.*;
import acm.util.*;
import java.io.*;
//import java.util.*;

@SuppressWarnings("serial")
public class LetterFrequency extends ConsoleProgram {

/** Runs the program */
	public void run() {
		initLetterCounts();
		countLettersInFile();
		printLetterCounts();
	}

/** Initializes the letter-frequency array */
	private void initLetterCounts() {
		letterCounts = new int[26];
		for (int i = 0; i < 26; i++) {
			letterCounts[i] = 0;
		}
	}

/** Reads the input file counting the letters */
	private void countLettersInFile() {
		BufferedReader rd = openInputFile("Enter name of text file: ");
		try {
			while (true) {
				String line = rd.readLine();
				if (line == null) break;
				for (int i = 0; i < line.length(); i++) {
					char ch = line.charAt(i);
					if (Character.isLetter(ch)) {
						letterCounts[Character.toUpperCase(ch) - 'A']++;
					}
				}
			}
		} catch (IOException ex) {
			throw new ErrorException(ex);
		}
	}
 	
/** Displays the table of letter counts */
	private void printLetterCounts() {
		println("Letter counts:");
		for (char ch = 'A'; ch <= 'Z'; ch++) {
			int index = ch - 'A';
			if (letterCounts[index] > 0) {
				println(ch + " " + letterCounts[index]);
			}
		}
	}

/**
 * Asks the user for the name of an input file and returns a
 * BufferedReader attached to its contents.  If the file does
 * not exist, the user is given another chance to try.
 */ 
	private BufferedReader openInputFile(String prompt) {
		BufferedReader rd = null;
		while (rd == null) {
			try {
				String name = readLine(prompt);
				rd = new BufferedReader(new FileReader(name));
			} catch (IOException ex) {
				println("Can't open that file.");
			}
		}
		return rd;
	}

/** Private instance variables */
	private int[] letterCounts;

}
	
