package demos;

/*
 * List.java
 * --------------
 * Uses arrays to manage a list of characters, 
 * including garbage collection
 */

import acm.program.*;

@SuppressWarnings("serial")
public class List extends ConsoleProgram {

	private static final int MAX_ITEMS = 6;

	private char[] theData = new char[MAX_ITEMS]; // the characters
	private int[] nextIndex = new int[MAX_ITEMS]; // pointers

	private int firstFreeIndex, firstDataIndex;

	public void run() {
		String nextCommand;

		initializeData();

		while (true) {
			displayData();
			nextCommand = readLine("Command (add, remove, quit): ")
					.toUpperCase();
			if (nextCommand.startsWith("A")) {
				addDatum(nextCommand.charAt(1));
			} else if (nextCommand.startsWith("R")) {
				removeData(nextCommand.charAt(1));
			} else if (nextCommand.startsWith("Q")) {
				break;
			}
		}
		println("Goodbye.");
	}

	/* Make the free list be everything and the data list be empty */
	private void initializeData() {
		firstFreeIndex = 0;

		for (int i = 0; i < MAX_ITEMS - 1; i++)
			nextIndex[i] = i + 1;
		nextIndex[MAX_ITEMS - 1] = -1;

		firstDataIndex = -1;
	}

	/* Add a value to the data list if possible */
	private void addDatum(char c) {
		if (firstFreeIndex == -1) {
			println("The list is full -- impossible to add a value.");
		} else {
			int temp = firstFreeIndex;
			firstFreeIndex = nextIndex[firstFreeIndex];
			nextIndex[temp] = firstDataIndex;
			firstDataIndex = temp;
			theData[firstDataIndex] = c;
		}
	}

	/* Remove all instances of a value from the data list if possible */
	private void removeData(char c) {
		int temp, pretemp;
		
		if (firstDataIndex == -1) {
			println("The list is empty -- impossible to remove a value.");
		
		} else {
			temp = firstDataIndex;

			/* remove occurrences of the value at the head of the list,
			 * updating the first data-index in the process
			 */
			while (temp == firstDataIndex) {
				if (temp == -1) break;
				if (theData[temp] == c) {
					println("Found the value at the beginning.");
					firstDataIndex = nextIndex[temp];
					nextIndex[temp] = firstFreeIndex;
					firstFreeIndex = temp;
					temp = firstDataIndex;
				} else {
					temp = nextIndex[firstDataIndex];
				}
			}
			/* Now either temp = -1 or nextIndex[firstDataIndex] = temp */
			pretemp = firstDataIndex;
			while (temp != -1) {
				if (theData[temp] == c) {
					println("Found the value, not at the beginning.");
					nextIndex[pretemp] = nextIndex[temp];
					nextIndex[temp] = firstFreeIndex;
					firstFreeIndex = temp;
					temp = nextIndex[pretemp];
				} else {
					pretemp = temp;
					temp = nextIndex[temp];
				}
			}
		}

	}

	private void displayData() {
		int index;
		println();
		
		print("The data are: {");
		index = firstDataIndex;
		while (index != -1) {
			print(theData[index] + "(@" + index + ")");
			if (nextIndex[index] != -1) print(", ");
			index = nextIndex[index];
		}
		println("}.");

		print("The free positions are: {");
		index = firstFreeIndex;
		while (index != -1) {
			print(index);
			if (nextIndex[index] != -1) print(", ");
			index = nextIndex[index];
		}
		println("}.");
		
		println();
	}

}
