/*
 * File: TrigraphFrequency.java
 * ----------------------------
 * This program builds a table showing the frequency of trigraphs
 * (three-letter sequences) in a data file.
 */

package demos;

import acm.program.*;
import acm.util.*;
import java.io.*;
import java.util.*;

@SuppressWarnings("serial")
public class TrigraphFrequency extends ConsoleProgram {

/** Runs the program */
	public void run() {
		trigraphs = new HashMap();
		countTrigraphs();
		printTrigraphCounts();
	}

/** Reads the input file counting the letters */
	private void countTrigraphs() {
		BufferedReader rd = openInputFile("Enter name of text file: ");
		try {
			while (true) {
				String line = rd.readLine();
				if (line == null) break;
				line = line.toUpperCase();
				for (int i = 0; i < line.length() - 2; i++) {
					String trigraph = line.substring(i, i + 3);
					if (containsOnlyLetters(trigraph)) {
						Integer counter = (Integer) trigraphs.get(trigraph);
						if (counter == null) {
							counter = new Integer(1);
						} else {
							counter = new Integer(counter.intValue() + 1);
						}
						trigraphs.put(trigraph, counter);
					}
				}
			}
		} catch (IOException ex) {
			throw new ErrorException(ex);
		}
	}
 	
/** Displays the table of letter counts */
	private void printTrigraphCounts() {
		for (Iterator i = trigraphs.keySet().iterator(); i.hasNext(); ) {
			String key = (String) i.next();
			Integer value = (Integer) trigraphs.get(key);
			println(key + " = " + value.intValue());
		}
	}

/** Checks if a string contains only letters */
	private boolean containsOnlyLetters(String str) {
		for (int i = 0; i < str.length(); i++) {
			if (!Character.isLetter(str.charAt(i))) return false;
		}
		return true;
	}

/**
 * Asks the user for the name of an input file and returns a
 * BufferedReader attached to its contents.  If the file does
 * not exist, the user is given another chance to try.
 */ 
	private BufferedReader openInputFile(String prompt) {
		BufferedReader rd = null;
		while (rd == null) {
			try {
				String name = readLine(prompt);
				rd = new BufferedReader(new FileReader(name));
			} catch (IOException ex) {
				println("Can't open that file.");
			}
		}
		return rd;
	}

/** Private instance variables */
	private HashMap trigraphs;
	
}
	


